@@ -2,14 +2,15 @@ use std::sync::{Arc, Mutex};
 use eframe::wgpu::AdapterInfo;
 use crate::egui::{self, RichText, util::History};
 use crate::{BASE, MEDIUM, HISTORY_LENGTH};
-use crate::{GuiAppData, util::*, fl};
+use crate::{GuiAppData, GuiGpuMetrics, util::*, fl};
+use crate::tab_gui::{MainTab, InfoTab};
 use egui_plot::{Corner, Legend, Line, Plot, PlotPoint, PlotPoints};
 
 use libamdgpu_top::{
-    AMDGPU::RasErrorCount,
+    AMDGPU::{GpuMetrics, MetricsInfo, RasErrorCount},
     DevicePath,
     PCI,
-    stat::{FdInfoSortType, PerfCounter},
+    stat::{FdInfoSortType, PerfCounter, Sensors},
 };
 
 const SPACING: [f32; 2] = [16.0; 2];
@@ -34,6 +35,9 @@ pub struct MyApp {
     pub no_pc: bool,
     pub pause: bool,
     pub full_fdinfo_list: bool,
+    pub tab_gui: bool,
+    pub main_tab: MainTab,
+    pub info_tab: InfoTab,
 }
 
 pub fn grid(ui: &mut egui::Ui, v: &[(&str, &str)]) {
@@ -84,14 +88,14 @@ impl MyApp {
 
                     let points: PlotPoints = history.iter()
                         .map(|(i, val)| [i, val as f64]).collect();
-                    let line = Line::new(points).fill(0.0);
+                    let line = Line::new(pc_index.name.clone(), points).fill(0.0);
 
                     default_plot(&pc_index.name)
                         .allow_scroll(false)
                         .include_y(0.0)
                         .include_y(100.0)
                         .label_formatter(label_fmt)
-                        .auto_bounds([true, false].into())
+                        .auto_bounds([true, false])
                         .height(SENSORS_HEIGHT / 2.0)
                         .width(SENSORS_WIDTH)
                         .show(ui, |plot_ui| plot_ui.line(line));
@@ -124,7 +128,7 @@ impl MyApp {
             .allow_scroll(false)
             .include_y(max as f64)
             .label_formatter(label_fmt)
-            .auto_bounds([true, false].into())
+            .auto_bounds([true, false])
             .height(PLOT_HEIGHT)
             .width(PLOT_WIDTH.min(ui.available_width()))
             .legend(Legend::default().position(Corner::LeftTop))
@@ -133,7 +137,7 @@ impl MyApp {
                     (vram, fl!("vram")),
                     (gtt, fl!("gtt"))
                 ] {
-                    plot_ui.line(Line::new(PlotPoints::new(usage)).name(name));
+                    plot_ui.line(Line::new(name, PlotPoints::new(usage)));
                 }
             });
     }
@@ -164,7 +168,7 @@ impl MyApp {
             format!("{:.1}s : {name} {:.0}%", val.x, val.y)
         };
 
-        let [mut gfx, mut compute, mut dma, mut dec, mut enc, mut media, mut vpe] = [0; 7]
+        let [mut gfx, mut compute, mut dma, mut dec, mut enc, mut vcnu, mut vpe] = [0; 7]
             .map(|_| Vec::<[f64; 2]>::with_capacity(HISTORY_LENGTH.end));
 
         for (i, usage) in self.buf_data.history.fdinfo_history.iter() {
@@ -173,7 +177,7 @@ impl MyApp {
             dma.push([i, usage.dma as f64]);
 
             if has_vcn_unified {
-                media.push([i, usage.media as f64]);
+                vcnu.push([i, usage.vcn_unified as f64]);
             } else {
                 dec.push([i, usage.total_dec as f64]);
                 enc.push([i, usage.total_enc as f64]);
@@ -189,7 +193,7 @@ impl MyApp {
             .include_y(100.0)
             .show_axes([false, true])
             .label_formatter(label_fmt)
-            .auto_bounds([true, false].into())
+            .auto_bounds([true, false])
             .height(PLOT_HEIGHT)
             .width(PLOT_WIDTH.min(ui.available_width()))
             .legend(Legend::default().position(Corner::LeftTop))
@@ -199,23 +203,27 @@ impl MyApp {
                     (compute, fl!("compute")),
                     (dma, fl!("dma")),
                 ] {
-                    plot_ui.line(Line::new(PlotPoints::new(usage)).name(name));
+                    plot_ui.line(Line::new(name, PlotPoints::new(usage)));
                 }
 
                 if has_vcn_unified {
-                    plot_ui.line(Line::new(PlotPoints::new(media)).name(fl!("media")));
+                    plot_ui.line(Line::new(fl!("vcn_unified"), PlotPoints::new(vcnu)));
                 } else {
-                    plot_ui.line(Line::new(PlotPoints::new(dec)).name(fl!("decode")));
-                    plot_ui.line(Line::new(PlotPoints::new(enc)).name(fl!("encode")));
+                    plot_ui.line(Line::new(fl!("decode"), PlotPoints::new(dec)));
+                    plot_ui.line(Line::new(fl!("encode"), PlotPoints::new(enc)));
                 }
 
                 if has_vpe {
-                    plot_ui.line(Line::new(PlotPoints::new(vpe)).name(fl!("vpe")));
+                    plot_ui.line(Line::new(fl!("vpe"), PlotPoints::new(vpe)));
                 }
             });
     }
 
     pub fn egui_fdinfo_list(&mut self, ui: &mut egui::Ui, has_vcn_unified: bool, has_vpe: bool) {
+        {
+            let vis = ui.visuals_mut();
+            vis.striped = true;
+        }
         egui::Grid::new("fdinfo").show(ui, |ui| {
             ui.style_mut().override_font_id = Some(MEDIUM);
             ui.label(rt_base(format!("{:^15}", fl!("name")))).highlight();
@@ -229,7 +237,7 @@ impl MyApp {
                 (fl!("gfx"), 5, FdInfoSortType::GFX, true),
                 (fl!("compute"), 9, FdInfoSortType::Compute, true),
                 (fl!("dma"), 5, FdInfoSortType::DMA, true),
-                (fl!("media"), 5, FdInfoSortType::MediaEngine, has_vcn_unified),
+                (fl!("vcn_unified"), 11, FdInfoSortType::VCNU, has_vcn_unified),
                 (fl!("decode"), 9, FdInfoSortType::Decode, !has_vcn_unified),
                 (fl!("encode"), 9, FdInfoSortType::Encode, !has_vcn_unified),
                 (fl!("vpe"), 5, FdInfoSortType::VPE, has_vpe),
@@ -264,7 +272,7 @@ impl MyApp {
                 ui.label(format!("{:5} {mib}", pu.usage.vram_usage >> 10));
                 ui.label(format!("{:5} {mib}", pu.usage.gtt_usage >> 10));
                 for usage in [
-                    pu.cpu_usage,
+                    pu.usage.cpu,
                     pu.usage.gfx,
                     pu.usage.compute,
                     pu.usage.dma,
@@ -273,7 +281,7 @@ impl MyApp {
                 }
 
                 if has_vcn_unified {
-                    ui.label(format!("{:3} %", pu.usage.media));
+                    ui.label(format!("{:3} %", pu.usage.vcn_unified));
                 } else {
                     ui.label(format!("{:3} %", pu.usage.total_dec));
                     ui.label(format!("{:3} %", pu.usage.total_enc));
@@ -330,12 +338,12 @@ impl MyApp {
             |ui| self.egui_fdinfo_plot(ui, has_vcn_unified, has_vpe),
         );
 
-        ui.toggle_value(&mut self.full_fdinfo_list, fl!("full_fdinfo_list"));
+        ui.checkbox(&mut self.full_fdinfo_list, fl!("full_fdinfo_list"));
 
         if self.full_fdinfo_list || (proc_len != 0 && proc_len < 8) {
             self.egui_fdinfo_list(ui, has_vcn_unified, has_vpe);
         } else {
-            egui::ScrollArea::vertical()
+            egui::ScrollArea::both()
                 .auto_shrink([false, false])
                 .min_scrolled_height(FDINFO_LIST_HEIGHT)
                 .show(ui, |ui| self.egui_fdinfo_list(ui, has_vcn_unified, has_vpe));
@@ -367,6 +375,14 @@ impl MyApp {
                     self.buf_data.device_info.max_mem_clk,
                     fl!("mhz"),
                 ),
+                (
+                    &self.buf_data.history.sensors_history.fclk,
+                    sensors.fclk_dpm.as_ref().map(|f| f.current_mhz),
+                    "FCLK",
+                    sensors.fclk_dpm.as_ref().map(|f| f.min_mhz).unwrap_or(0),
+                    sensors.fclk_dpm.as_ref().map(|f| f.max_mhz).unwrap_or(3000),
+                    fl!("mhz"),
+                ),
                 (
                     &self.buf_data.history.sensors_history.vddgfx,
                     sensors.vddgfx,
@@ -425,7 +441,7 @@ impl MyApp {
                     };
                     let points: PlotPoints = history.iter()
                         .map(|(i, val)| [i, val as f64]).collect();
-                    let line = Line::new(points).fill(0.0);
+                    let line = Line::new(label.to_string(), points).fill(0.0);
 
                     Plot::new(label)
                         .allow_zoom(false)
@@ -434,7 +450,7 @@ impl MyApp {
                         .include_y(max)
                         .show_axes(false)
                         .label_formatter(label_fmt)
-                        .auto_bounds([true, false].into())
+                        .auto_bounds([true, false])
                         .height(SENSORS_HEIGHT)
                         .width(SENSORS_WIDTH)
                         .show(ui, |plot_ui| plot_ui.line(line));
@@ -448,6 +464,8 @@ impl MyApp {
             self.egui_temp_plot(ui);
         });
 
+        self.egui_core_freq_plot(ui, sensors);
+
         if let Some(cur) = sensors.current_link {
             let min_max = if let [Some(min), Some(max)] = [sensors.min_dpm_link, sensors.max_dpm_link] {
                 format!(
@@ -514,12 +532,12 @@ impl MyApp {
                     .iter()
                     .map(|(i, val)| [i, val as f64])
                     .collect();
-                let line = Line::new(points).fill(0.0);
+                let line = Line::new(label.to_string(), points).fill(0.0);
 
                 default_plot(label)
                     .include_y(max)
                     .label_formatter(label_fmt)
-                    .auto_bounds([true, true].into())
+                    .auto_bounds([true, true])
                     .height(SENSORS_HEIGHT)
                     .width(SENSORS_WIDTH)
                     .show(ui, |plot_ui| plot_ui.line(line));
@@ -539,13 +557,13 @@ impl MyApp {
                     .iter()
                     .map(|(i, val)| [i, val as f64])
                     .collect();
-                let line = Line::new(points).fill(0.0);
+                let line = Line::new(label.to_string(), points).fill(0.0);
 
                 default_plot(label)
                     .include_y(0)
                     .include_y(100)
                     .label_formatter(label_fmt)
-                    .auto_bounds([true, true].into())
+                    .auto_bounds([true, true])
                     .height(SENSORS_HEIGHT)
                     .width(SENSORS_WIDTH)
                     .show(ui, |plot_ui| plot_ui.line(line));
@@ -574,14 +592,14 @@ impl MyApp {
             }
 
             [
-                Line::new(PlotPoints::new(sent_history)).name(&fl_sent),
-                Line::new(PlotPoints::new(rec_history)).name(&fl_rec),
+                Line::new(fl_sent.clone(), PlotPoints::new(sent_history)),
+                Line::new(fl_rec.clone(), PlotPoints::new(rec_history)),
             ]
         };
 
         default_plot("pcie_bw plot")
             .label_formatter(label_fmt)
-            .auto_bounds([true, true].into())
+            .auto_bounds([true, true])
             .height(PLOT_HEIGHT)
             .width(PLOT_WIDTH.min(ui.available_width()))
             .legend(Legend::default().position(Corner::LeftTop))
@@ -621,7 +639,7 @@ impl MyApp {
                 .map(|(i, act)| [i, act as f64])
                 .collect();
 
-            Line::new(PlotPoints::new(v)).name(name)
+            Line::new(name, PlotPoints::new(v))
         });
 
         default_plot("activity plot")
@@ -640,6 +658,45 @@ impl MyApp {
             });
     }
 
+    pub fn egui_core_freq_plot(&self, ui: &mut egui::Ui, sensors: &Sensors) {
+        if sensors.all_cpu_core_freq_info.is_empty() {
+            return;
+        }
+
+        {
+            let mut s = String::with_capacity(128);
+            let Ok(_) = sensors.print_all_cpu_core_cur_freq(
+                &mut s,
+                "\nCPU Core freq (MHz)",
+                false,
+            ) else { return };
+            ui.label(s);
+            ui.end_row();
+        }
+
+        let all_core_freq: Vec<Vec<[f64; 2]>> = self.buf_data.history.sensors_history.core_freq
+            .iter()
+            .map(|history| history.iter().map(|(i, mhz)| [i, mhz as f64]).collect())
+            .collect();
+        let label_fmt = |name: &str, val: &PlotPoint| {
+            format!("{:.1}s : {name} {:.0} MHz", val.x, val.y)
+        };
+
+        Plot::new("Core Freq Plot")
+            .allow_zoom(false)
+            .allow_scroll(false)
+            .show_axes([false, true])
+            .label_formatter(label_fmt)
+            .auto_bounds([true, true])
+            .height(PLOT_HEIGHT)
+            .width(PLOT_WIDTH.min(ui.available_width() - 100.0))
+            .legend(Legend::default().position(Corner::LeftTop))
+            .show(ui, |plot_ui| for (i, freq) in all_core_freq.into_iter().enumerate() {
+                plot_ui.line(Line::new(format!("Core{i}"), PlotPoints::new(freq)))
+            });
+        ui.label(""); // \n
+    }
+
     pub fn egui_core_power_plot(&self, ui: &mut egui::Ui) {
         let Some(core_power_mw) = &self.buf_data.history.core_power_mw else { return };
         let all_core_power_mw: Vec<Vec<[f64; 2]>> = core_power_mw
@@ -655,12 +712,12 @@ impl MyApp {
             .allow_scroll(false)
             .show_axes([false, true])
             .label_formatter(label_fmt)
-            .auto_bounds([true, true].into())
+            .auto_bounds([true, true])
             .height(PLOT_HEIGHT)
             .width(PLOT_WIDTH.min(ui.available_width() - 100.0))
             .legend(Legend::default().position(Corner::LeftTop))
             .show(ui, |plot_ui| for (i, mw) in all_core_power_mw.into_iter().enumerate() {
-                plot_ui.line(Line::new(PlotPoints::new(mw)).name(format!("Core{i}")))
+                plot_ui.line(Line::new(format!("Core{i}"), PlotPoints::new(mw)))
             });
     }
 
@@ -684,7 +741,7 @@ impl MyApp {
             .width(PLOT_WIDTH.min(ui.available_width() - 100.0))
             .legend(Legend::default().position(Corner::LeftTop))
             .show(ui, |plot_ui| for (i, temp_c) in all_core_temp.into_iter().enumerate() {
-                plot_ui.line(Line::new(PlotPoints::new(temp_c)).name(format!("Core{i}")))
+                plot_ui.line(Line::new(format!("Core{i}"), PlotPoints::new(temp_c)))
             });
     }
 
@@ -716,15 +773,75 @@ impl MyApp {
                     (dclk1, "DCLK1"),
                 ] {
                     if !clk.is_empty() {
-                        plot_ui.line(Line::new(PlotPoints::new(clk)).name(name))
+                        plot_ui.line(Line::new(name, PlotPoints::new(clk)))
                     }
                 }
             });
     }
 
+    pub fn egui_gpu_metrics(&self, ui: &mut egui::Ui) {
+        let Some(metrics) = &self.buf_data.stat.metrics else { return };
+        let Some(header) = metrics.get_header() else { return };
+
+        let header = format!(
+            "{} v{}.{}",
+            fl!("gpu_metrics"),
+            header.format_revision,
+            header.content_revision
+        );
+
+        match metrics {
+            GpuMetrics::V1_0(_) |
+            GpuMetrics::V1_1(_) |
+            GpuMetrics::V1_2(_) |
+            GpuMetrics::V1_3(_) |
+            GpuMetrics::V1_4(_) |
+            GpuMetrics::V1_5(_) => {
+                collapsing(ui, &header, true, |ui| metrics.v1_ui(ui));
+                collapsing_plot(ui, &fl!("vclk_dclk_plot"), true, |ui| self.egui_vclk_dclk_plot(ui));
+            },
+            /* APU */
+            GpuMetrics::V2_0(_) |
+            GpuMetrics::V2_1(_) |
+            GpuMetrics::V2_2(_) |
+            GpuMetrics::V2_3(_) |
+            GpuMetrics::V2_4(_) => {
+                collapsing(ui, &header, true, |ui| {
+                    metrics.v2_ui(ui);
+                    if self.buf_data.history.core_temp.is_some() {
+                        collapsing_plot(
+                            ui,
+                            &fl!("cpu_temp_plot"),
+                            true,
+                            |ui| self.egui_core_temp_plot(ui),
+                        );
+                    }
+                    if self.buf_data.history.core_power_mw.is_some() {
+                        collapsing_plot(
+                            ui,
+                            &fl!("cpu_power_plot"),
+                            true,
+                            |ui| self.egui_core_power_plot(ui),
+                        );
+                    }
+                    collapsing_plot(ui, &fl!("vclk_dclk_plot"), true, |ui| self.egui_vclk_dclk_plot(ui));
+                });
+            },
+            /* APU */
+            GpuMetrics::V3_0(_) => {
+                collapsing(ui, &header, true, |ui| {
+                    metrics.v3_ui(ui);
+                    collapsing_plot(ui, &fl!("cpu_temp_plot"), true, |ui| self.egui_core_temp_plot(ui));
+                    collapsing_plot(ui, &fl!("cpu_power_plot"), true, |ui| self.egui_core_power_plot(ui));
+                    collapsing_plot(ui, &fl!("vclk_dclk_plot"), true, |ui| self.egui_vclk_dclk_plot(ui));
+                });
+            },
+            _ => {},
+        }
+    }
 }
 
-fn default_plot(id: &str) -> Plot {
+fn default_plot(id: &str) -> Plot<'_> {
     Plot::new(id)
         .allow_zoom(false)
         .allow_scroll(false)
