@@ -38,6 +38,7 @@ pub struct HistoryData {
 pub struct SensorsHistory {
     pub sclk: History<u32>,
     pub mclk: History<u32>,
+    pub fclk: History<u32>,
     pub vddgfx: History<u32>,
     pub vddnb: History<u32>,
     pub edge_temp: History<i64>,
@@ -47,22 +48,25 @@ pub struct SensorsHistory {
     pub input_power: History<u32>,
     pub fan_rpm: History<u32>,
     pub tctl: History<i64>,
+    pub core_freq: Vec<History<u32>>,
 }
 
 impl SensorsHistory {
     pub fn new() -> Self {
-        let [sclk, mclk, vddgfx, vddnb, average_power, input_power, fan_rpm] = [0; 7]
+        let [sclk, mclk, fclk, vddgfx, vddnb, average_power, input_power, fan_rpm] = [0; 8]
             .map(|_| History::new(HISTORY_LENGTH, f32::INFINITY));
         let [edge_temp, junction_temp, memory_temp, tctl] = [0;4]
             .map(|_| History::new(HISTORY_LENGTH, f32::INFINITY));
+        let core_freq = vec![History::new(HISTORY_LENGTH, f32::INFINITY); 64];
 
-        Self { sclk, mclk, vddgfx, vddnb, edge_temp, junction_temp, memory_temp, average_power, input_power, fan_rpm, tctl }
+        Self { sclk, mclk, fclk, vddgfx, vddnb, edge_temp, junction_temp, memory_temp, average_power, input_power, fan_rpm, tctl, core_freq }
     }
 
     pub fn add(&mut self, sec: f64, sensors: &Sensors) {
         for (history, val) in [
             (&mut self.sclk, sensors.sclk),
             (&mut self.mclk, sensors.mclk),
+            (&mut self.fclk, sensors.fclk_dpm.as_ref().map(|f| f.current_mhz)),
             (&mut self.vddgfx, sensors.vddgfx),
             (&mut self.vddnb, sensors.vddnb),
             (&mut self.average_power, sensors.average_power.as_ref().map(|power| power.value)),
@@ -85,6 +89,10 @@ impl SensorsHistory {
         if let Some(tctl_val) = sensors.tctl {
             self.tctl.add(sec, tctl_val / 1000);
         }
+
+        for (freq, freq_history) in sensors.all_cpu_core_freq_info.iter().zip(self.core_freq.iter_mut()) {
+            freq_history.add(sec, freq.cur);
+        }
     }
 }
 
@@ -94,7 +102,6 @@ impl Default for SensorsHistory {
     }
 }
 
-
 #[derive(Clone)]
 pub struct GuiAppData {
     pub stat: AppAmdgpuTopStat,
@@ -173,10 +180,11 @@ impl GuiAppData {
     }
 
     pub fn update_history(&mut self, secs: f64, no_pc: bool) {
+        let metrics = self.stat.metrics.as_ref();
         if let Some(arc_pcie_bw) = &self.stat.arc_pcie_bw {
             let lock = arc_pcie_bw.try_lock();
-            if let Ok(pcie_bw) = lock {
-                if let (Some(sent), Some(rec), Some(mps)) = (
+            if let Ok(pcie_bw) = lock
+                && let (Some(sent), Some(rec), Some(mps)) = (
                     pcie_bw.sent,
                     pcie_bw.received,
                     pcie_bw.max_payload_size,
@@ -185,7 +193,6 @@ impl GuiAppData {
                     let rec = (rec * mps as u64) >> 20;
                     self.history.pcie_bw_history.add(secs, (sent, rec));
                 }
-            }
         }
 
         if !no_pc {
@@ -199,10 +206,10 @@ impl GuiAppData {
             }
         }
 
-        if let Some(thr_val) = self.stat.metrics.as_ref().and_then(|m| m.get_indep_throttle_status()) {
-            if thr_val != 0 {
-                self.history.throttling_history.add(secs, ThrottleStatus::new(thr_val));
-            }
+        if let Some(thr_val) = metrics.and_then(|m| m.get_indep_throttle_status())
+            && thr_val != 0
+        {
+            self.history.throttling_history.add(secs, ThrottleStatus::new(thr_val));
         }
 
         if let Some(ref mut sensors) = self.stat.sensors {
@@ -211,7 +218,7 @@ impl GuiAppData {
 
         self.history.vram_history.add(secs, self.stat.vram_usage.0.vram.heap_usage);
         self.history.gtt_history.add(secs, self.stat.vram_usage.0.gtt.heap_usage);
-        self.history.fdinfo_history.add(secs, self.stat.fdinfo.fold_fdinfo_usage());
+        self.history.fdinfo_history.add(secs, self.stat.fdinfo.fold_fdinfo_usage().0);
 
         if let Some(gfx) = self.stat.activity.gfx {
             self.history.gfx_activity.add(secs, gfx);
@@ -223,39 +230,35 @@ impl GuiAppData {
             self.history.media_activity.add(secs, media);
         }
 
-        if let Some(vclk) = self.stat.metrics.as_ref().and_then(|m| m.get_current_vclk()) {
+        if let Some(vclk) = metrics.and_then(|m| m.get_current_vclk()) {
             self.history.vclk.add(secs, vclk);
         }
-        if let Some(dclk) = self.stat.metrics.as_ref().and_then(|m| m.get_current_dclk()) {
+        if let Some(dclk) = metrics.and_then(|m| m.get_current_dclk()) {
             self.history.dclk.add(secs, dclk);
         }
-        if let Some(vclk1) = self.stat.metrics.as_ref().and_then(|m| m.get_current_vclk1()) {
+        if let Some(vclk1) = metrics.and_then(|m| m.get_current_vclk1()) {
             self.history.vclk1.add(secs, vclk1);
         }
-        if let Some(dclk1) = self.stat.metrics.as_ref().and_then(|m| m.get_current_dclk1()) {
+        if let Some(dclk1) = metrics.and_then(|m| m.get_current_dclk1()) {
             self.history.dclk1.add(secs, dclk1);
         }
 
-        if let Some(ref mut history_core_temp) = self.history.core_temp {
-            if let Some(core_temp) = self.stat.metrics
-                .as_ref()
+        if let Some(ref mut history_core_temp) = self.history.core_temp
+            && let Some(core_temp) = metrics
                 .and_then(|m| gpu_metrics_util::check_temp_array(m.get_temperature_core()))
             {
                 for (history, v) in history_core_temp.iter_mut().zip(core_temp.iter()) {
                     history.add(secs, *v);
                 }
             }
-        }
 
-        if let Some(ref mut history_core_power_mw) = self.history.core_power_mw {
-            if let Some(core_power_mw) = self.stat.metrics
-                .as_ref()
+        if let Some(ref mut history_core_power_mw) = self.history.core_power_mw
+            && let Some(core_power_mw) = metrics
                 .and_then(|m| gpu_metrics_util::check_power_clock_array(m.get_average_core_power()))
             {
                 for (history, v) in history_core_power_mw.iter_mut().zip(core_power_mw.iter()) {
                     history.add(secs, *v);
                 }
             }
-        }
     }
 }
