@@ -276,10 +276,12 @@ impl GuiInfo for AppDeviceInfo {
         } else {
             fl!("not_supported")
         };
+        let vram_vendor = self.memory_vendor.clone().unwrap_or(fl!("unknown"));
 
         grid(ui, &[
             (&fl!("vram_type"), &self.ext_info.get_vram_type().to_string()),
             (&fl!("vram_bit_width"), &format!("{}-{}", self.ext_info.vram_bit_width, fl!("bit"))),
+            (&fl!("vram_vendor"), &vram_vendor),
             (&fl!("vram_size"), &format!("{} {}", self.memory_info.vram.total_heap_size >> 20, fl!("mib"))),
             (&fl!("memory_clock"), &format!("{}-{} {}", self.min_mem_clk, self.max_mem_clk, fl!("mhz"))),
             (&fl!("resizable_bar"), &re_bar),
