@@ -2,7 +2,7 @@ use libamdgpu_top::{
     DevicePath,
     stat,
     xdna,
-    AMDGPU::{GpuMetrics, MetricsInfo},
+    AMDGPU::{GpuMetrics, MetricsInfo, FW_VERSION::FwVer, HW_IP::HwIpInfo, IpDieEntry, IpHwId, IpHwInstance},
     VramUsage,
     PCI,
     ConnectorInfo,
@@ -62,6 +62,7 @@ impl OutputJson for Sensors {
         for (label, val, unit) in [
             ("GFX_SCLK", self.sclk, "MHz"),
             ("GFX_MCLK", self.mclk, "MHz"),
+            ("FCLK", self.fclk_dpm.as_ref().map(|f| f.current_mhz), "MHz"),
             ("VDDNB", self.vddnb, "mV"),
             ("VDDGFX", self.vddgfx, "mV"),
             ("Fan", self.fan_rpm, "RPM"),
@@ -91,17 +92,33 @@ impl OutputJson for Sensors {
         }
 
         for (label, temp, unit) in [
-            ("Edge Temperature", &self.edge_temp, "C"),
-            ("Junction Temperature", &self.junction_temp, "C"),
-            ("Memory Temperature", &self.memory_temp, "C"),
+            ("Edge", &self.edge_temp, "C"),
+            ("Junction", &self.junction_temp, "C"),
+            ("Memory", &self.memory_temp, "C"),
         ] {
             m.insert(
-                label.to_string(),
+                format!("{label} Temperature"),
                 temp.as_ref().map_or(Value::Null, |temp| json!({
                     "value": temp.current,
                     "unit": unit,
                 })),
             );
+
+            m.insert(
+                format!("{label} Critical Temperature"),
+                temp.as_ref().map_or(Value::Null, |temp| json!({
+                    "value": temp.critical,
+                    "unit": unit,
+                })),
+            );
+
+            m.insert(
+                format!("{label} Emergency Temperature"),
+                temp.as_ref().map_or(Value::Null, |temp| json!({
+                    "value": temp.emergency,
+                    "unit": unit,
+                })),
+            );
         }
 
         m.insert(
@@ -112,6 +129,21 @@ impl OutputJson for Sensors {
             })),
         );
 
+        let all_cpu_core_freq: Vec<Value> = self.all_cpu_core_freq_info
+            .iter()
+            .map(|freq_info| json!({
+                "core_id": freq_info.core_id,
+                "thread_id": freq_info.thread_id,
+                "min_freq": freq_info.min,
+                "cur_freq": freq_info.cur,
+                "max_freq": freq_info.max,
+            }))
+            .collect();
+        m.insert(
+            "CPU Core freq".to_string(),
+            Value::Array(all_cpu_core_freq),
+        );
+
         m.insert(
             "PCIe Link Speed".to_string(),
             self.current_link.map_or(Value::Null, |link| link.json()),
@@ -131,8 +163,12 @@ impl OutputJson for Sensors {
     }
 }
 
-impl OutputJson for FdInfoUsage {
-    fn json(&self) -> Value {
+pub trait FdInfoJson {
+    fn usage_json(&self, has_vcn: bool, has_vcn_unified: bool, has_vpe: bool) -> Value;
+}
+
+impl FdInfoJson for FdInfoUsage {
+    fn usage_json(&self, has_vcn: bool, has_vcn_unified: bool, has_vpe: bool) -> Value {
         let mut sub = Map::new();
         sub.insert(
             "VRAM".to_string(),
@@ -150,60 +186,16 @@ impl OutputJson for FdInfoUsage {
         );
 
         for (label, val) in [
-            ("GFX", self.gfx),
-            ("Compute", self.compute),
-            ("DMA", self.dma),
-            ("Decode", self.total_dec),
-            ("Encode", self.total_enc),
-            ("Media", self.media),
-            ("VCN_JPEG", self.vcn_jpeg),
-            ("VPE", self.vpe),
-        ] {
-            sub.insert(
-                label.to_string(),
-                json!({
-                    "value": val,
-                    "unit": "%",
-                })
-            );
-        }
-
-        sub.into()
-    }
-}
-
-pub trait FdInfoJson {
-    fn usage_json(&self, has_vcn: bool, has_vcn_unified: bool, has_vpe: bool) -> Value;
-}
-
-impl FdInfoJson for ProcUsage {
-    fn usage_json(&self, has_vcn: bool, has_vcn_unified: bool, has_vpe: bool) -> Value {
-        let mut sub = Map::new();
-        sub.insert(
-            "VRAM".to_string(),
-            json!({
-                "value": self.usage.vram_usage >> 10,
-                "unit": "MiB",
-            }),
-        );
-        sub.insert(
-            "GTT".to_string(),
-            json!({
-                "value": self.usage.gtt_usage >> 10,
-                "unit": "MiB",
-            }),
-        );
-
-        for (label, val) in [
-            ("GFX", Some(self.usage.gfx)),
-            ("Compute", Some(self.usage.compute)),
-            ("DMA", Some(self.usage.dma)),
-            ("Decode", if !has_vcn_unified { Some(self.usage.total_dec) } else { None }),
-            ("Encode", if !has_vcn_unified { Some(self.usage.total_enc) } else { None }),
-            ("CPU", Some(self.cpu_usage)),
-            ("Media", Some(self.usage.media)),
-            ("VCN_JPEG", if has_vcn { Some(self.usage.vcn_jpeg) } else { None }),
-            ("VPE", if has_vpe { Some(self.usage.vpe) } else { None }),
+            ("CPU", Some(self.cpu)),
+            ("GFX", Some(self.gfx)),
+            ("Compute", Some(self.compute)),
+            ("DMA", Some(self.dma)),
+            ("Decode", if !has_vcn_unified { Some(self.total_dec) } else { None }),
+            ("Encode", if !has_vcn_unified { Some(self.total_enc) } else { None }),
+            ("Media", Some(self.media)),
+            ("VCN_JPEG", if has_vcn { Some(self.vcn_jpeg) } else { None }),
+            ("VPE", if has_vpe { Some(self.vpe) } else { None }),
+            ("VCN_Unified", if has_vcn_unified { Some(self.vcn_unified) } else { None }),
         ] {
             sub.insert(
                 label.to_string(),
@@ -218,9 +210,15 @@ impl FdInfoJson for ProcUsage {
             );
         }
 
+        sub.into()
+    }
+}
+
+impl FdInfoJson for ProcUsage {
+    fn usage_json(&self, has_vcn: bool, has_vcn_unified: bool, has_vpe: bool) -> Value {
         json!({
             "name": self.name,
-            "usage": sub,
+            "usage": self.usage.usage_json(has_vcn, has_vcn_unified, has_vpe),
         })
     }
 }
@@ -527,3 +525,60 @@ impl OutputJson for drmModeModeInfo {
         })
     }
 }
+
+impl OutputJson for HwIpInfo {
+    fn json(&self) -> Value {
+        json!({
+            "ip_type": self.ip_type.to_string(),
+            "ip_count": self.count,
+            "major": self.info.hw_ip_version_major,
+            "minor": self.info.hw_ip_version_minor,
+            "queues": self.info.num_queues(),
+        })
+    }
+}
+
+impl OutputJson for FwVer {
+    fn json(&self) -> Value {
+        json!({
+            "fw_type": self.fw_type.to_string(),
+            "ip_instance": self.ip_instance,
+            "index": self.index,
+            "version": self.version,
+            "feature": self.feature,
+        })
+    }
+}
+
+impl OutputJson for IpHwInstance {
+    fn json(&self) -> Value {
+        json!({
+            "hw_id": self.hw_id.to_string(),
+            "num_instance": self.num_instance,
+            "major": self.major,
+            "minor": self.minor,
+            "revision": self.revision,
+            "harvest": self.harvest,
+            "num_base_addresses": self.num_base_addresses,
+            "base_address": self.base_address,
+        })
+    }
+}
+
+impl OutputJson for IpHwId {
+    fn json(&self) -> Value {
+        json!({
+            "hw_id": self.hw_id.to_string(),
+            "instances": self.instances.iter().map(|i| i.json()).collect::<Vec<Value>>(),
+        })
+    }
+}
+
+impl OutputJson for IpDieEntry {
+    fn json(&self) -> Value {
+        json!({
+            "die_id": self.die_id,
+            "ip_hw_ids": self.ip_hw_ids.iter().map(|i| i.json()).collect::<Vec<Value>>(),
+        })
+    }
+}
