@@ -282,6 +282,8 @@ impl TuiApp {
 
         if let Some(metrics) = &self.app_amdgpu_top.stat.metrics {
             let _ = self.layout.gpu_metrics_view.print_gpu_metrics(metrics);
+        } else {
+            self.layout.gpu_metrics_view.text.clear();
         }
 
         if !self.no_pc {
