@@ -11,9 +11,7 @@ use crate::{Text, AppTextView};
 
 const GPU_NAME_LEN: usize = 25;
 const LINE_LEN: usize = 150;
-const THR_LEN: usize = 42;
-const ECC_LABEL: &str = "ECC_UnCorr.";
-const ECC_LEN: usize = ECC_LABEL.len()-2;
+const THR_LEN: usize = 57;
 const PROC_TITLE: &str = "Processes";
 
 use libamdgpu_top::app::AppAmdgpuTop;
@@ -27,23 +25,11 @@ struct SmiApp {
 }
 
 impl SmiApp {
-    pub fn new(app_amdgpu_top: AppAmdgpuTop, index: usize) -> Option<Self> {
-        let gfxoff_monitor = GfxoffMonitor::new(app_amdgpu_top.device_path.pci).ok();
-
-        Some(Self {
-            app_amdgpu_top,
-            index,
-            gfxoff_monitor,
-            fdinfo_view: Default::default(),
-            info_text: Default::default(),
-        })
-    }
-
     fn info_header() -> TextView {
         let text = format!(concat!(
             "GPU {name:<name_len$} {pad:10}|{pci:<16}|{vram:^18}|\n",
             "SCLK    MCLK    VDDGFX  Power           | GFX% UMC%Media%|{gtt:^18}|\n",
-            "Temp    {fan:<7} {tctl}  {ecc} {thr:<THR_LEN$}|"
+            "GPU/MEM_T  {fan:<7} {thr:<THR_LEN$}|"
             ),
             name = "Name",
             name_len = GPU_NAME_LEN,
@@ -52,8 +38,6 @@ impl SmiApp {
             gtt = " GTT Usage",
             pad = "",
             fan = "Fan",
-            tctl = "Tctl",
-            ecc = "ECC_UnCorr.",
             thr = "Throttle_Status",
             THR_LEN = THR_LEN,
         );
@@ -61,21 +45,6 @@ impl SmiApp {
         TextView::new(text).no_wrap()
     }
 
-    fn info_text(&mut self) -> TextView {
-        TextView::new_with_content(self.info_text.content.clone()).no_wrap()
-    }
-
-    fn fdinfo_panel(&self) -> Panel<TextView> {
-        let text = TextView::new_with_content(self.fdinfo_view.text.content.clone()).no_wrap();
-        Panel::new(text)
-            .title(format!(
-                "#{:<2} {}",
-                self.index,
-                self.app_amdgpu_top.device_info.marketing_name,
-            ))
-            .title_position(HAlign::Left)
-    }
-
     fn update_info_text(&mut self) -> Result<(), std::fmt::Error> {
         let sensors = self.app_amdgpu_top.stat.sensors.as_ref();
         self.info_text.clear();
@@ -153,10 +122,14 @@ impl SmiApp {
             gt = self.app_amdgpu_top.stat.vram_usage.0.gtt.total_heap_size >> 20,
         )?;
 
-        if let Some(temp) = sensors.and_then(|s| s.edge_temp.as_ref()) {
-            write!(self.info_text.buf, " {:>3}C ", temp.current)?;
+        if let Some(temp) = sensors.and_then(|s| s.junction_temp.as_ref().or(s.edge_temp.as_ref())) {
+            if let Some(mem_temp) = sensors.and_then(|s| s.memory_temp.as_ref()) {
+                write!(self.info_text.buf, "{:>3}C/{:>3}C", temp.current, mem_temp.current)?;
+            } else {
+                write!(self.info_text.buf, "{:>3}C/___C", temp.current)?;
+            }
         } else {
-            write!(self.info_text.buf, " ___C ")?;
+            write!(self.info_text.buf, "___C/___C")?;
         }
 
         if let Some(fan_rpm) = sensors.and_then(|s| s.fan_rpm) {
@@ -165,18 +138,6 @@ impl SmiApp {
             write!(self.info_text.buf, "  ____RPM ")?;
         }
 
-        if let Some(tctl) = sensors.and_then(|s| s.tctl) {
-            write!(self.info_text.buf, " {:>3}C ", tctl / 1000)?;
-        } else {
-            write!(self.info_text.buf, " ___C ")?;
-        }
-
-        if let Some(ecc) = &self.app_amdgpu_top.stat.memory_error_count {
-            write!(self.info_text.buf, "[{:>ECC_LEN$}] ", ecc.uncorrected)?;
-        } else {
-            write!(self.info_text.buf, "[{:>ECC_LEN$}] ", "N/A")?;
-        }
-
         if let Some(thr) = self.app_amdgpu_top.stat.metrics.as_ref().and_then(|m| m.get_throttle_status_info()) {
             let thr = format!("{:?}", thr.get_all_throttler());
             write!(
@@ -185,10 +146,33 @@ impl SmiApp {
                 thr.get(..THR_LEN).unwrap_or_else(|| &thr)
             )?;
         } else {
-            write!(
-                self.info_text.buf,
-                "{:<THR_LEN$}|",
-                "N/A",
+            let pci_power_state = if sensors.is_some_and(|s| s.is_idle) {
+                sensors.and_then(|sensors| sensors.pci_power_state.as_ref())
+            } else {
+                None
+            };
+            let s = if let Some(s) = pci_power_state {
+                &format!("N/A ({s})")
+            } else {
+                "N/A"
+            };
+
+            write!(self.info_text.buf, "{s:<THR_LEN$}|")?;
+        }
+
+        if self.app_amdgpu_top.device_info.is_apu && let Some(sensors) = sensors {
+            writeln!(self.info_text.buf)?;
+
+            if let Some(tctl) = sensors.tctl {
+                write!(self.info_text.buf, " Tctl:{:>3}C", tctl / 1000)?;
+            } else {
+                write!(self.info_text.buf, " Tctl:___C")?;
+            }
+
+            sensors.print_all_cpu_core_cur_freq(
+                &mut self.info_text.buf,
+                " CPU freq (100MHz)",
+                true,
             )?;
         }
 
@@ -202,7 +186,11 @@ impl SmiApp {
 
         let _ = self.fdinfo_view.print_fdinfo(
             &mut self.app_amdgpu_top.stat.fdinfo,
-            FdInfoSortType::default(),
+            if self.app_amdgpu_top.device_info.is_apu {
+                FdInfoSortType::GTT
+            } else {
+                FdInfoSortType::default()
+            },
             false,
         );
 
@@ -222,21 +210,12 @@ impl SuspendedSmiApp {
     fn new(device_path: DevicePath, index: usize) -> Self {
         let mut info_text: Text = Default::default();
 
-        if let [Some(did), Some(rid)] = [device_path.device_id, device_path.revision_id] {
-            let _ = writeln!(
-                info_text.buf,
-                "#{index:<2} [{name:<20} ({did:#X}:{rid:#X})]| {pci}   | Suspended",
-                name = device_path.device_name,
-                pci = device_path.pci,
-            );
-        } else {
-            let _ = writeln!(
-                info_text.buf,
-                "#{index:<2} [{name:<20}]| {pci}   | Suspended",
-                name = device_path.device_name,
-                pci = device_path.pci,
-            );
-        }
+        let _ = writeln!(
+            info_text.buf,
+            "#{index:<2} [{name:<GPU_NAME_LEN$}]         | {pci}   | Suspended        |",
+            name = device_path.device_name,
+            pci = device_path.pci,
+        );
 
         info_text.set();
 
@@ -287,20 +266,11 @@ pub fn run_smi(
     }: UiArgs,
 ) {
     let sample = Sampling::low();
-    let (vec_app, suspended) = AppAmdgpuTop::create_app_and_suspended_list(
-        &device_path_list,
-        &Default::default(),
-    );
-    let mut vec_app: Vec<_> = vec_app
-        .into_iter()
+    let mut vec_app: Vec<SmiApp> = Vec::with_capacity(device_path_list.len());
+    let mut sus_app_devices: Vec<_> = device_path_list
+        .iter()
         .enumerate()
-        .filter_map(|(i, app)| SmiApp::new(app, i))
-        .collect();
-    let app_len = vec_app.len();
-    let mut sus_app_devices: Vec<_> = suspended
-        .into_iter()
-        .enumerate()
-        .map(|(i, device_path)| SuspendedSmiApp::new(device_path.clone(), app_len+i))
+        .map(|(i, device_path)| SuspendedSmiApp::new(device_path.clone(), i))
         .collect();
 
     let mut siv = cursive::default();
@@ -311,11 +281,6 @@ pub fn run_smi(
             let mut info = LinearLayout::vertical()
                 .child(SmiApp::info_header())
                 .child(TextView::new_with_content(line.clone()).no_wrap());
-            for app in vec_app.iter_mut() {
-                app.update(&sample);
-                info.add_child(app.info_text());
-                info.add_child(TextView::new_with_content(line.clone()).no_wrap());
-            }
             for sus_app in sus_app_devices.iter_mut() {
                 info.add_child(sus_app.info_text());
                 info.add_child(TextView::new_with_content(line.clone()).no_wrap());
@@ -325,9 +290,6 @@ pub fn run_smi(
         }
         {
             let mut proc = LinearLayout::vertical();
-            for app in &vec_app {
-                proc.add_child(app.fdinfo_panel());
-            }
             for sus_app in sus_app_devices.iter_mut() {
                 proc.add_child(sus_app.fdinfo_panel());
             }
@@ -358,6 +320,14 @@ pub fn run_smi(
 
     let cb_sink = siv.cb_sink().clone();
 
+    {
+        extract_active_devices(&mut vec_app, &mut sus_app_devices);
+
+        for app in vec_app.iter_mut() {
+            app.update(&sample);
+        }
+    }
+
     std::thread::spawn(move || loop {
         std::thread::sleep(sample.to_duration()); // 1s
 
@@ -365,19 +335,23 @@ pub fn run_smi(
             app.update(&sample);
         }
 
-        sus_app_devices.retain(|sus_app| {
-            let is_active = sus_app.device_path.check_if_device_is_active();
-
-            if is_active {
-                let Some(smi_app) = sus_app.to_smi_app() else { return true };
-                vec_app.push(smi_app);
-            }
-
-            !is_active
-        });
+        extract_active_devices(&mut vec_app, &mut sus_app_devices);
 
         cb_sink.send(Box::new(cursive::Cursive::noop)).unwrap();
     });
 
     siv.run();
 }
+
+fn extract_active_devices(vec_app: &mut Vec<SmiApp>, sus_app_devices: &mut Vec<SuspendedSmiApp>) {
+    sus_app_devices.retain(|sus_app| {
+        let is_active = sus_app.device_path.check_if_device_is_active();
+
+        if is_active {
+            let Some(smi_app) = sus_app.to_smi_app() else { return true };
+            vec_app.push(smi_app);
+        }
+
+        !is_active
+    });
+}
