@@ -16,7 +16,7 @@ const COMPUTE_LABEL: &str = "COMP";
 const DMA_LABEL: &str = "DMA";
 const DEC_LABEL: &str = "DEC";
 const ENC_LABEL: &str = "ENC";
-const VCN_LABEL: &str = "VCN";
+const VCN_UNIFIED_LABEL: &str = "VCNU";
 const VPE_LABEL: &str = "VPE";
 const KFD_LABEL: &str = "KFD";
 // const UVD_ENC_LABEL: &str = "UVD (ENC)";
@@ -37,19 +37,19 @@ impl AppTextView {
 
         write!(
             self.text.buf,
-            " {proc_name:<PROC_NAME_LEN$}|{pid:^PID_MAX_LEN$}|{KFD_LABEL}|{VRAM_LABEL:^6}|{GTT_LABEL:^6}|{CPU_LABEL:^4}|{GFX_LABEL:^4}|{COMPUTE_LABEL:^4}|{DMA_LABEL:^4}",
+            " {proc_name:<PROC_NAME_LEN$}|{pid:^PID_MAX_LEN$}|{KFD_LABEL}|{VRAM_LABEL:^6}|{GTT_LABEL:^6}|{CPU_LABEL:^4}|{GFX_LABEL:^4}|{COMPUTE_LABEL:^4}|{DMA_LABEL:^4}|",
             proc_name = "Name",
             pid = "PID",
         )?;
 
         if stat.has_vcn_unified {
-            write!(self.text.buf, "|{VCN_LABEL:^4}|")?;
+            write!(self.text.buf, "{VCN_UNIFIED_LABEL:^4}|")?;
         } else {
-            write!(self.text.buf, "|{DEC_LABEL:^4}|{ENC_LABEL:^4}|")?;
+            write!(self.text.buf, "{DEC_LABEL:^4}|{ENC_LABEL:^4}|")?;
         }
 
         if stat.has_vpe {
-            write!(self.text.buf, "|{VPE_LABEL:^4}|")?;
+            write!(self.text.buf, "{VPE_LABEL:^4}|")?;
         }
 
         writeln!(self.text.buf)?;
@@ -81,7 +81,7 @@ impl AppTextView {
                 gtt = pu.usage.gtt_usage >> 10,
             )?;
 
-            write!(self.text.buf, "{:>3}%|", pu.cpu_usage)?;
+            write!(self.text.buf, "{:>3}%|", pu.usage.cpu)?;
 
             for (usage, label_len) in [
                 (pu.usage.gfx, GFX_LABEL.len()),
@@ -92,7 +92,7 @@ impl AppTextView {
             }
 
             if stat.has_vcn_unified {
-                write!(self.text.buf, "{:>3}%|", pu.usage.media)?;
+                write!(self.text.buf, "{:>3}%|", pu.usage.vcn_unified)?;
             } else {
                 write!(self.text.buf, "{:>3}%|", pu.usage.total_dec)?;
                 write!(self.text.buf, "{:>3}%|", pu.usage.total_enc)?;
@@ -174,7 +174,7 @@ impl AppTextView {
     pub fn cb_sort_by_media(siv: &mut cursive::Cursive) {
         {
             let mut opt = siv.user_data::<Opt>().unwrap().lock().unwrap();
-            opt.fdinfo_sort = FdInfoSortType::MediaEngine;
+            opt.fdinfo_sort = FdInfoSortType::VCNU;
         }
     }
 }
