@@ -3,12 +3,11 @@ use crate::AMDGPU::{DeviceHandle, GPU_INFO, GpuMetrics, MetricsInfo, RasBlock, R
 use crate::{AppDeviceInfo, DevicePath, stat, xdna, VramUsage, has_vcn, has_vcn_unified, has_vpe};
 use stat::{FdInfoStat, GpuActivity, Sensors, PcieBw, PerfCounter, ProcInfo};
 use xdna::XdnaFdInfoStat;
-use std::mem::ManuallyDrop;
 use std::sync::{Arc, Mutex};
 use std::time::Duration;
 
 pub struct AppAmdgpuTop {
-    amdgpu_dev: ManuallyDrop<Option<DeviceHandle>>,
+    amdgpu_dev: DeviceHandle,
     pub device_info: AppDeviceInfo,
     pub device_path: DevicePath,
     pub xdna_device_path: Option<DevicePath>,
@@ -16,7 +15,8 @@ pub struct AppAmdgpuTop {
     pub stat: AppAmdgpuTopStat,
     buf_interval: Duration,
     no_drop_device_handle: bool,
-    dynamic_no_pc: bool, // to transition the APU into GFXOFF state
+    dynamic_no_pc: bool, // to transition APU into GFXOFF state or dGPU into D3Hot state
+    to_d3hot: bool,
 }
 
 #[derive(Clone)]
@@ -35,6 +35,40 @@ pub struct AppAmdgpuTopStat {
     pub memory_error_count: Option<RasErrorCount>,
 }
 
+impl AppAmdgpuTopStat {
+    // Workaround:
+    // Raphael/Granite Ridge APU (SMU v13.0.5) reports very low input power. (7.00 - 15.00 mW)
+    //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/2321
+    //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/3999
+    fn workaround_for_smu_v13_0_5(&mut self) -> Option<()> {
+        let input_power = self.sensors.as_mut()?.input_power.as_mut()?;
+
+        if input_power.value == 0 {
+            let avg_socket_power = self.metrics.as_ref()?.get_average_socket_power()?;
+            input_power.value = avg_socket_power;
+        }
+
+        None
+    }
+
+    // Workaround:
+    // `*_get_fan_speed_rpm` is missing in `drivers/gpu/drm/amd/pm/swsmu/smu14/smu_v14_0_2_ppt.c`,
+    // so SMU v14.0.2/3 dose not have `fan1_input` in `hwmon`.
+    //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/4034
+    //     ref: https://github.com/Umio-Yasuno/amdgpu_top/issues/123
+    fn workaround_fan_rpm_for_smu_v14(&mut self) -> Option<()> {
+        let sensors = self.sensors.as_mut()?;
+        if sensors.fan_rpm.is_none() {
+            sensors.fan_rpm = self.metrics
+                .as_ref()?
+                .get_current_fan_speed()
+                .map(|fan_rpm| fan_rpm as u32);
+        }
+
+        None
+    }
+}
+
 pub struct AppOption {
     pub pcie_bw: bool,
 }
@@ -73,7 +107,7 @@ impl AppAmdgpuTop {
             let (device_path, other_sus_devs) = suspended_devices.split_first().unwrap();
             // wake up
             let amdgpu_dev = device_path.init().unwrap();
-            let app = AppAmdgpuTop::new(
+            let app = Self::new(
                 amdgpu_dev,
                 device_path.clone(),
                 &Default::default(),
@@ -194,7 +228,7 @@ impl AppAmdgpuTop {
         }
 
         Some(Self {
-            amdgpu_dev: ManuallyDrop::new(Some(amdgpu_dev)),
+            amdgpu_dev,
             device_info,
             device_path,
             xdna_device_path,
@@ -216,11 +250,12 @@ impl AppAmdgpuTop {
             buf_interval: Duration::ZERO,
             no_drop_device_handle,
             dynamic_no_pc: false,
+            to_d3hot: false,
         })
     }
 
     pub fn update(&mut self, interval: Duration) {
-        {
+        if interval != Duration::ZERO {
             let fdinfo_lock = self.stat.arc_proc_index.try_lock();
             let xdna_fdinfo_lock = self.stat.arc_xdna_proc_index.try_lock();
 
@@ -238,28 +273,29 @@ impl AppAmdgpuTop {
             }
         }
         {
-            let proc_len = self.stat.fdinfo.proc_usage.len();
-            let pre_activity = &self.stat.activity;
+            let no_process_using_vram = self.stat.fdinfo.no_process_using_vram();
 
-            // running GPU process is only "amdgpu_top"
             // TODO: those checks may not be enough
-            if proc_len == 1
-                && self.amdgpu_dev.is_some()
+            if self.stat.activity.is_all_idling()
+                && !self.to_d3hot
                 && !self.no_drop_device_handle
                 && !self.device_info.is_apu
-                && pre_activity.is_all_idling()
             {
-                unsafe { ManuallyDrop::drop(&mut self.amdgpu_dev); }
-                self.amdgpu_dev = ManuallyDrop::new(None);
-            } else if proc_len > 1
-                && self.amdgpu_dev.is_none()
+                self.stat.vram_usage.update_usage(&self.amdgpu_dev);
+                self.stat.vram_usage.update_usable_heap_size(&self.amdgpu_dev);
+
+                self.to_d3hot = true;
+            } else if self.to_d3hot
+                && !no_process_using_vram
                 && self.device_path.check_if_device_is_active()
             {
-                self.amdgpu_dev = ManuallyDrop::new(self.device_path.init().ok());
+                self.to_d3hot = false;
             }
+
+            self.dynamic_no_pc = no_process_using_vram;
         }
 
-        if self.amdgpu_dev.is_none() {
+        if self.to_d3hot {
             if let Some(ref mut sensors) = self.stat.sensors {
                 sensors.update_for_idle();
             }
@@ -268,66 +304,34 @@ impl AppAmdgpuTop {
             return;
         };
 
-        if self.stat.metrics.is_some()
-        || (self.stat.metrics.is_none() && self.stat.sensors.is_none())
-        {
+        if self.device_info.supports_gpu_metrics {
             self.stat.metrics = GpuMetrics::get_from_sysfs_path(&self.device_info.sysfs_path).ok();
         }
 
-        if let Some(dev) = self.amdgpu_dev.as_ref() {
-            self.stat.vram_usage.update_usage(dev);
-            self.stat.vram_usage.update_usable_heap_size(dev);
+        {
+            self.stat.vram_usage.update_usage(&self.amdgpu_dev);
+            self.stat.vram_usage.update_usable_heap_size(&self.amdgpu_dev);
 
             if let Some(ref mut sensors) = self.stat.sensors {
-                sensors.update(dev);
+                sensors.update(&self.amdgpu_dev);
             } else {
                 self.stat.sensors = Sensors::new(
-                    dev,
+                    &self.amdgpu_dev,
                     &self.device_info.pci_bus,
                     &self.device_info.ext_info,
                 );
             }
-
         }
 
-        // Workaround:
-        // Raphael/Granite Ridge APU (SMU v13.0.5) reports very low input power. (7.00 - 15.00 mW)
-        //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/2321
-        //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/3999
         if self.device_info.smu_ip_version.is_some_and(|ip_ver| ip_ver == (13, 0, 5))
-        && self.device_info.smc_fw_version.is_some_and(|ver| ver >= 0x624F00)
         {
-            if let Some(input_power) = self.stat.sensors
-                .as_mut()
-                .and_then(|s| s.input_power.as_mut())
-            {
-                if input_power.value == 0 {
-                    if let Some(v) = self.stat.metrics
-                        .as_ref()
-                        .and_then(|m| m.get_average_socket_power())
-                    {
-                        input_power.value = v;
-                    }
-                }
-            }
+            self.stat.workaround_for_smu_v13_0_5();
         }
 
-        // Workaround:
-        // `*_get_fan_speed_rpm` is missing in `drivers/gpu/drm/amd/pm/swsmu/smu14/smu_v14_0_2_ppt.c`,
-        // so SMU v14.0.2/3 dose not have `fan1_input` in `hwmon`.
-        //     ref: https://gitlab.freedesktop.org/drm/amd/-/issues/4034
-        //     ref: https://github.com/Umio-Yasuno/amdgpu_top/issues/123
         if self.device_info.smu_ip_version
             .is_some_and(|ip_ver| ip_ver == (14, 0, 2) || ip_ver == (14, 0, 3))
         {
-            if let Some(sensors) = self.stat.sensors.as_mut() {
-                if sensors.fan_rpm.is_none() {
-                    sensors.fan_rpm = self.stat.metrics
-                        .as_ref()
-                        .and_then(|m| m.get_current_fan_speed())
-                        .map(|fan_rpm| fan_rpm as u32);
-                }
-            }
+            self.stat.workaround_fan_rpm_for_smu_v14();
         }
 
         if self.stat.memory_error_count.is_some() {
@@ -343,20 +347,17 @@ impl AppAmdgpuTop {
             &self.stat.metrics,
         );
 
-        self.dynamic_no_pc = self.device_info.is_apu && self.stat.activity.is_gfx_idling();
-
         if self.stat.activity.media.is_none() || self.stat.activity.media == Some(0) {
-            self.stat.activity.media = self.stat.fdinfo.fold_fdinfo_usage().media.try_into().ok();
+            let (proc_usage, _, _, _) = self.stat.fdinfo.fold_fdinfo_usage();
+            self.stat.activity.media = proc_usage.media.try_into().ok();
         }
     }
 
     pub fn update_pc(&mut self) {
-        if self.dynamic_no_pc { return }
+        if self.dynamic_no_pc || self.to_d3hot { return }
 
-        if let Some(dev) = self.amdgpu_dev.as_ref() {
-            self.stat.grbm.read_reg(dev);
-            self.stat.grbm2.read_reg(dev);
-        }
+        self.stat.grbm.read_reg(&self.amdgpu_dev);
+        self.stat.grbm2.read_reg(&self.amdgpu_dev);
     }
 
     pub fn clear_pc(&mut self) {
@@ -370,8 +371,6 @@ impl AppAmdgpuTop {
     }
 
     pub fn get_drm_version_struct(&mut self) -> Option<drmVersion> {
-        self.amdgpu_dev
-            .as_ref()
-            .and_then(|dev| dev.get_drm_version_struct().ok())
+        self.amdgpu_dev.get_drm_version_struct().ok()
     }
 }
