@@ -1,3 +1,4 @@
+use std::fmt::{self, Write};
 use std::io::Read;
 use std::path::{Path, PathBuf};
 use std::fs;
@@ -8,6 +9,8 @@ use libdrm_amdgpu_sys::{
         GPU_INFO,
         ASIC_NAME,
         DeviceHandle,
+        DpmClockRange,
+        DpmClockType,
         HwmonTemp,
         HwmonTempType,
         SENSOR_INFO::SENSOR_TYPE,
@@ -15,7 +18,7 @@ use libdrm_amdgpu_sys::{
         PowerProfile,
     },
 };
-use super::{parse_hwmon, HwmonPower, PowerType};
+use super::{CpuFreqInfo, parse_hwmon, HwmonPower, PowerType};
 
 #[derive(Clone, Debug)]
 pub struct Sensors {
@@ -44,8 +47,12 @@ pub struct Sensors {
     pub fan_max_rpm: Option<u32>,
     pub pci_power_state: Option<String>,
     pub power_profile: Option<PowerProfile>,
+    pub fclk_dpm: Option<DpmClockRange>,
+    // pub socclk_dpm: Option<DpmClockRange>,
     k10temp_tctl_path: Option<PathBuf>,
     pub tctl: Option<i64>, // CPU Temp.
+    pub all_cpu_core_freq_info: Vec<CpuFreqInfo>,
+    pub is_idle: bool,
 }
 
 impl Sensors {
@@ -111,12 +118,16 @@ impl Sensors {
         let fan_rpm = parse_hwmon(hwmon_path.join("fan1_input"));
         let fan_max_rpm = parse_hwmon(hwmon_path.join("fan1_max"));
         let gpu_port_path = pci_bus.get_gpu_pcie_port_bus().get_sysfs_path();
-        let pci_power_state = fs::read_to_string(gpu_port_path.join("power_state"))
-            .ok()
-            .map(|mut s| {
-                s.pop();
-                s
-            });
+        let pci_power_state = if !is_apu {
+            fs::read_to_string(gpu_port_path.join("power_state"))
+                .ok()
+                .map(|mut s| {
+                    s.pop();
+                    s
+                })
+        } else {
+            None
+        };
         let power_profile = PowerProfile::get_current_profile_from_sysfs(&sysfs_path);
         let k10temp_path = if is_apu {
             Self::find_k10temp_path()
@@ -130,6 +141,15 @@ impl Sensors {
             None
         };
 
+        let all_cpu_core_freq_info = if is_apu {
+            CpuFreqInfo::get_all_cpu_core_freq_info()
+        } else {
+            Vec::new()
+        };
+
+        let fclk_dpm = DpmClockRange::from_sysfs(DpmClockType::FCLK, &sysfs_path);
+        // let socclk_dpm = DpmClockRange::from_sysfs(DpmClockType::SOCCLK, &sysfs_path);
+
         Some(Self {
             hwmon_path,
             sysfs_path,
@@ -156,8 +176,12 @@ impl Sensors {
             gpu_port_path,
             pci_power_state,
             power_profile,
+            fclk_dpm,
+            // socclk_dpm,
             k10temp_tctl_path,
             tctl,
+            all_cpu_core_freq_info,
+            is_idle: false,
         })
     }
 
@@ -170,30 +194,22 @@ impl Sensors {
             self.bus_info.get_current_link_info()
         };
 
-        for temp in [&mut self.edge_temp, &mut self.junction_temp, &mut self.memory_temp] {
-            let Some(temp) = temp else { continue };
-            temp.update(&self.hwmon_path);
-        }
+        let hwmon_path = &self.hwmon_path;
 
-        if self.average_power.is_some() {
-            self.average_power = HwmonPower::from_hwmon_path_with_type(
-                &self.hwmon_path,
-                PowerType::Average,
-            );
-        }
-
-        if self.input_power.is_some() {
-            self.input_power = HwmonPower::from_hwmon_path_with_type(
-                &self.hwmon_path,
-                PowerType::Input,
-            );
-        }
+        self.edge_temp = HwmonTemp::from_hwmon_path(hwmon_path, HwmonTempType::Edge);
+        self.junction_temp = HwmonTemp::from_hwmon_path(hwmon_path, HwmonTempType::Junction);
+        self.memory_temp = HwmonTemp::from_hwmon_path(hwmon_path, HwmonTempType::Memory);
+        self.power_cap = PowerCap::from_hwmon_path(hwmon_path);
+        self.average_power = HwmonPower::from_hwmon_path_with_type(hwmon_path, PowerType::Average);
+        self.input_power = HwmonPower::from_hwmon_path_with_type(hwmon_path, PowerType::Input);
 
         self.fan_rpm = parse_hwmon(self.hwmon_path.join("fan1_input"));
         self.power_profile = PowerProfile::get_current_profile_from_sysfs(&self.sysfs_path);
         self.power_cap = PowerCap::from_hwmon_path(&self.hwmon_path);
         self.update_pci_power_state();
         self.update_tctl();
+        self.update_all_cpu_core_cur_freq();
+        self.is_idle = false;
     }
 
     pub fn update_for_idle(&mut self) {
@@ -209,17 +225,21 @@ impl Sensors {
         self.vddgfx = None;
         self.fan_rpm = None;
         self.power_profile = None;
+        self.fclk_dpm = None;
+        self.is_idle = true;
 
         self.update_pci_power_state();
     }
 
     pub fn update_pci_power_state(&mut self) {
-        self.pci_power_state = fs::read_to_string(self.gpu_port_path.join("power_state"))
-            .ok()
-            .map(|mut s| {
-                s.pop(); // trim `\n`
-                s
-            });
+        if !self.is_apu {
+            self.pci_power_state = fs::read_to_string(self.gpu_port_path.join("power_state"))
+                .ok()
+                .map(|mut s| {
+                    s.pop(); // trim `\n`
+                    s
+                });
+        }
     }
 
     pub fn update(&mut self, amdgpu_dev: &DeviceHandle) {
@@ -228,6 +248,8 @@ impl Sensors {
         self.mclk = amdgpu_dev.sensor_info(SENSOR_TYPE::GFX_MCLK).ok();
         self.vddnb = amdgpu_dev.sensor_info(SENSOR_TYPE::VDDNB).ok();
         self.vddgfx = amdgpu_dev.sensor_info(SENSOR_TYPE::VDDGFX).ok();
+        self.fclk_dpm = DpmClockRange::from_sysfs(DpmClockType::FCLK, &self.sysfs_path);
+        // self.socclk_dpm = DpmClockRange::from_sysfs(DpmClockType::SOCCLK, &self.sysfs_path);
     }
 
     pub fn any_hwmon_power(&self) -> Option<HwmonPower> {
@@ -266,4 +288,42 @@ impl Sensors {
             self.tctl = Self::get_tctl(path);
         }
     }
+
+    fn update_all_cpu_core_cur_freq(&mut self) {
+        for freq_info in self.all_cpu_core_freq_info.iter_mut() {
+            freq_info.update_cur_freq();
+        }
+    }
+
+    pub fn print_all_cpu_core_cur_freq(
+        &self,
+        buf: &mut String,
+        label: &str,
+        divide_by_100: bool,
+    ) -> fmt::Result {
+        if self.all_cpu_core_freq_info.is_empty() { return Ok(()) }
+
+        write!(buf, "{label}: [")?;
+
+        for cpu_freq_info in &self.all_cpu_core_freq_info {
+            if divide_by_100 {
+                write!(
+                    buf,
+                    "{:>2},",
+                    cpu_freq_info.cur.div_ceil(100),
+                )?;
+            } else {
+                write!(
+                    buf,
+                    "{:>4},",
+                    cpu_freq_info.cur,
+                )?;
+            }
+        }
+
+        let _ = buf.pop();
+        write!(buf, "]")?;
+
+        Ok(())
+    }
 }
