@@ -30,26 +30,42 @@ fn main() {
         return;
     }
 
-    let (device_path_list, device_path) = {
-        let list = DevicePath::get_device_path_list();
+    fn get_list_and_selected_device_path(main_opt: &MainOpt)
+        -> (Vec<DevicePath>, DevicePath)
+    {
+        let mut list = DevicePath::get_device_path_list();
 
         if list.is_empty() {
             eprintln!("There are no the AMD GPU devices found.");
             panic!();
         }
 
-        let device_path = if main_opt.select_apu {
+        let selected_device_path = if main_opt.select_apu {
             select_apu(&list)
         } else {
-            from_main_opt(&main_opt, &list)
+            from_main_opt(main_opt, &list)
         };
 
         if main_opt.single_gpu {
-            (vec![device_path.clone()], device_path)
-        } else {
-            (list, device_path)
+            return (vec![selected_device_path.clone()], selected_device_path);
         }
-    };
+
+        if main_opt.select_apu || main_opt.pci.is_some() || main_opt.instance.is_some() {
+            let pos = list
+                .iter()
+                .position(|device_path| device_path.pci == selected_device_path.pci)
+                .unwrap();
+
+            if pos != 0 {
+                list.remove(pos);
+                list.insert(0, selected_device_path.clone());
+            }
+        }
+
+        (list, selected_device_path)
+    }
+
+    let (device_path_list, device_path) = get_list_and_selected_device_path(&main_opt);
 
     #[cfg(feature = "json")]
     if let AppMode::JSON = main_opt.app_mode { match main_opt.dump_mode {
@@ -141,6 +157,7 @@ fn main() {
         is_dark_mode: main_opt.is_dark_mode,
         hide_fdinfo: main_opt.hide_fdinfo,
         gui_wgpu_backend: main_opt.wgpu_backend,
+        tab_gui: main_opt.tab_gui,
     };
 
     match main_opt.app_mode {
@@ -152,7 +169,7 @@ fn main() {
             #[cfg(not(feature = "tui"))]
             {
                 eprintln!("\"tui\" feature is not enabled for this build.");
-                dump_info::dump(&ui_args.device_path, main_opt.opt_dump_mode);
+                dump_info::dump(&ui_args.selected_device_path, main_opt.opt_dump_mode);
             }
         },
         #[cfg(feature = "gui")]
@@ -207,12 +224,7 @@ pub fn device_list(list: &[DevicePath]) {
 
 pub fn from_main_opt(main_opt: &MainOpt, list: &[DevicePath]) -> DevicePath {
     if let Some(pci) = main_opt.pci {
-        DevicePath::try_from(pci).unwrap_or_else(|err| {
-            eprintln!("{err}");
-            eprintln!("pci_path: {pci:?}");
-            eprintln!("Device list: {list:#?}");
-            panic!();
-        })
+        list.iter().find(|device_path| device_path.pci == pci).unwrap().clone()
     } else if let Some(i) = main_opt.instance {
         list
             .get(i)
@@ -225,7 +237,7 @@ pub fn from_main_opt(main_opt: &MainOpt, list: &[DevicePath]) -> DevicePath {
             })
             .clone()
     } else {
-        list.iter().next().unwrap().clone()
+        list.first().unwrap().clone()
     }
 }
 
